

/*-------------------------------------------------------------------------------------------------------------- 
	MB and Bob Consulting
	Copyright 2006 - MB and Bob Consulting
	All Rights Reserved
 
--------------------------------------------------------------------------------------------------------------*/

/* BobsScriptLibrary.jsx
	Author: 		Bob Stucky
	Version:		1.0
	Dated:			02/07/2007
*/

BobsLib = {};
///
/// 	Classes
///

///
///	Handy Dandy log object
/// 	usage:
/// 		var log = new BobsLib.Log( pathOrFile, true );
/// 		log.writeln( "This is my first log entry" );
///		log.close();
///
	BobsLib.Log = function( logFile, includeDate ) {
		this.includeDate = typeof( includeDate ) == "boolean" ? includeDate : true;
		this.file = new File( logFile ); // note passing a file object or path to constructor yields a file
		this.file.parent.verify(); // this will ensure the folder containing the file exists
		if ( this.file.exists ) {
			this.file.open( "e" );
			this.file.seek( 0, 2 ); // pos should be end of text now
		} else {
			this.file.open( "w" );
		}
		this.debug = true;
		this.file.writeln( BobsLib.Log.getDate() + ": Log Session Opened");
	}
	BobsLib.Log.prototype.clearLog = function() {
		this.file.close();
		this.file.open( "w" );
		this.writeln( "Log Cleared..." );
	}
	BobsLib.Log.prototype.setDebug = function( debug ) {
		this.debug = debug;
	}
	BobsLib.Log.getDate = function() { // static method to get the standard date format for the log
		var d = new Date();
		if ( ( d.getYear() - 100 ) < 10 ) {
			var yearStr = "0" + new String( ( d.getYear() - 100 ) );
		} else {
			var yearStr = new String ( ( d.getYear() - 100 ) );
		}
		var ds = ( d.getMonth() + 1 ) + "/" + d.getDate() + "/" + yearStr + " " + d.getHours() + ":" + d.getMinutes() + ":" + d.getSeconds();
		return ds;
	 }
	BobsLib.Log.prototype.write = function( txt, overrideDebug ) {
		if ( !this.debug && !overrideDebug ) {
			return;
		}
		txt = txt == undefined ? "" : txt; // write nothing if nothing passed in (instead of "undefined")
		this.file.write( txt );
	}
	BobsLib.Log.prototype.writeln = function( txt, overrideDebug ) {
		if ( !this.debug && !overrideDebug ) {
			return;
		}
		txt = txt == undefined ? "" : txt;
		if ( this.includeDate ) {
			if ( txt == "" ) {
				this.file.writeln( txt );
			} else {
				this.file.writeln( BobsLib.Log.getDate() + ": " + txt );
			}
		} else {
			this.file.writeln( txt );
		}
	}
	BobsLib.Log.prototype.writeObject = function( obj, overrideDebug ) {
		if ( !this.debug && !overrideDebug ) {
			return;
		}
			this.writeln( obj.toString() );
	}
	BobsLib.Log.prototype.writeException = function( e, includeSource ) {
		var debugSetting = this.debug;
		this.debug = true;
		this.writeln( "--Exception!!" );
		this.writeln( "   Message: " + e.message );
		this.writeln( "   File: " + e.fileName );
		this.writeln( "   Line: " + e.line );
		this.writeln( "   Error Name: " + e.name );
		if ( e.source ) {	
			var srcArray = e.source.split( "\n" );
			var a = e.line - 10;
			var b = e.line + 10;
			var c = e.line - 1;
			for ( var i = a; ( i < b ) && ( i < srcArray.length ); i++ ) {
				if ( i == c ) {
					this.writeln( "   Source: (" + ( i + 1 ) + ") >> " + srcArray[ i ] );
				} else {
					this.writeln( "   Source: (" + ( i + 1 ) + ")    " + srcArray[ i ] );
				}
			}
		}
		this.writeln( "--End Exception" );
		this.debug = debugSetting;
	}
	BobsLib.Log.prototype.writeFormatted = function( array ) {
		if ( !this.debug ) {
			return;
		}
		if ( this.includeDate ) {
			this.file.write( BobsLib.Log.getDate() + ": " );
		} 
		for ( var i = 0; i < array.length; i++ ) {
			this.file.write( array[ i ] + "\t" );
		}
		this.file.writeln();
	}
	BobsLib.Log.prototype.close = function() {
		this.file.writeln( BobsLib.Log.getDate() +  ": Log Session Closed" );
		this.file.writeln();
		this.file.close();
	}
///
/// 	StringBuffer
/// 	Fastest way to concatenate strings in JavaScript
/// 	usage:
/// 		var sb = new BobsLib.StringBuffer();
///		sb.append( "The rain " );
/// 		sb.append( "in Spain..." );
///		var finalString = sb.toString();
///
	BobsLib.StringBuffer = function() {
		this.array = new Array();
		this.length = 0;
	}
///	note this is not done as a subclass of array as the length property 
///	would be calc'd using the array's methods, and would not be correct
	BobsLib.StringBuffer.prototype.toString = function( sep ) {
		if ( sep ) { // for example, use a newline char to have each snippet on its own line
			return this.array.join( sep );
		} else {
			return this.array.join( "" );
		}
	}
	BobsLib.StringBuffer.prototype.append = function( x ) {
		if ( x ) {
			this.array.push( x.toString() );
			this.length += x.toString().length;
		}
	}
	BobsLib.StringBuffer.prototype.clear = function() {
		this.array = new Array();
		this.length = 0;
	}
	BobsLib.StringBuffer.prototype.concat = function( sBuffer ) {
		this.array = this.array.concat( sBuffer.array );
		this.length += sBuffer.length;
	}
///
///	Hashtable
///	handy way to store name-value pairs - is serializable via toSource() and can be used with BridgeTalk
///	usage
///		var ht = new BobsLib.Hashtable();
///		ht.putValue( "Fred", "Flintstone" );
///		var out = ht.getValue( "Fred" );
///
///	for serialization: 
///	ht1 = eval( ht.toSource() );
///	out = ht1.getValue( "Fred" );
///  ht1 is an exact copy of ht, but it is a copy, NOT a reference to the same hashtable
///
BobsLib.Hashtable = function( caseSensitive ) {
	this.caseSensitive = caseSensitive || false;
	this.keyList = new Array( 0 );
	this.keyIndex = {};
	this.count = 0;
}
BobsLib.Hashtable.prototype._getKey = function( name ) {
	return this.caseSensitive ? new String( "htx%_" +  name ) : new String( "htx%_" +  name ).toLowerCase();
}
BobsLib.Hashtable.prototype.clone = function() {
	var t = new BobsLib.Hashtable( this.caseSensitive );
	for ( var i = 0; i < this.keyList.length; i++ ) {
		t.putValue( this.keyList[ i ], this.getValue( this.keyList[ i ] ) );
	}
	return t;
}
BobsLib.Hashtable.prototype.hasKey = function( name ) {
	return !this.isUniqueKey( this._getKey( name ) );
}
BobsLib.Hashtable.prototype.merge = function( ht ) {
	for ( var i = 0; i < ht.keyList.length; i++ ) {
		this.putValue( ht.keyList[ i ], ht.getValue( ht.keyList[ i ] ) );
	}
}
BobsLib.Hashtable.prototype.isUniqueKey = function( key ) {
	return this.keyIndex[ key ] == undefined;
}
BobsLib.Hashtable.prototype.putValue = function( name, value ) {
	if ( ( value instanceof Boolean ) || ( typeof value == "boolean" ) ) {
		value = value ? "true" : "false";
	}
	var key = this._getKey( name );
	this[ key ] = value;
	if ( this.isUniqueKey( key ) ) {
		this.keyIndex[ key ] = this.keyList.length;
		this.keyList.push( name );
		this.count++;
	} else {
		this.keyList[ this.keyIndex[ key ] ] = name;
	}
}			
BobsLib.Hashtable.prototype.keys = function()  {
	return this.keyList;
}
BobsLib.Hashtable.prototype.getValue = function( name ) {
	return this[ this._getKey( name ) ];
	try {
		if ( val ) {
			if ( ( val == "true" ) || ( val == "false" ) ) {
				val = eval( val );
			}
		}
	} catch ( e ) {
		var val = this[ this._getKey( name ) ];
	}
	return val;
}
BobsLib.Hashtable.prototype.getCount = function()  {
	return this.count;
}
BobsLib.Hashtable.prototype.remove = function( name )  {
	if ( this.getValue( name ) )  {
		var key = this._getKey( name );
		var idx = this.keyIndex[ key ];
		this.keyIndex[ key ] = undefined;
		this[ key ] = undefined;
		this.keyList.splice( idx, 1 );
		this.count--;
	}
}
BobsLib.Hashtable.prototype.toString = function() {
	return "[object BobsLib.Hashtable] " + this.keyList;
}
///
/// StackTable
/// creates a "stack" of hashtables, but returns stuff as if it's a hashtable
/// the current hashtable is the one that you are getting from and putting to
/// using "clone()" clones the current hashtable, places the old current table
/// on the stack. The cloned hashtable is now the current hashtable.
/// popping, replaces the current table with the one on top of the stack.
/// This widget is really handy when you have to keep track of the context
/// of a highly recursive operation where each level of the operation has
/// the potential to change the context for all downline operations. Cloning
/// ensures the current context is maintained, which can be modified by down line
/// operations. When the recursion unwinds back to where the clone() happened, popping
/// the stacktable will restore the context to what it was before the downline operations
///
BobsLib.StackTable = function( caseSensitive ) {
	this.caseSensitive = caseSensitive || false;
	this.current = new BobsLib.Hashtable( this.caseSensitive );
	this.stack = new Array();
	this.keyList = this.current.keyList;
	this.keyIndex = this.current.keyIndex;
	this.count = this.current.count;
}
BobsLib.StackTable.prototype.getValue = function( key ) {
	return this.current.getValue( key );
}
BobsLib.StackTable.prototype.putValue = function( key, value ) {
	this.current.putValue( key, value );
	this.count = this.current.count;
}
BobsLib.StackTable.prototype.pop = function() {		
	this.current = this.stack.pop();
	this.count = this.current.count;
	this.keyList = this.current.keyList;
	this.keyIndex = this.current.keyIndex;
}
BobsLib.StackTable.prototype.remove = function( key ) {
	this.current.remove( key );
	this.count = this.current.count;
	this.keyList = this.current.keyList;
	this.keyIndex = this.current.keyIndex;
}
BobsLib.StackTable.prototype.clone = function() {
	this.stack.push( this.current );
	var t = this.current.clone();
	this.current = t;
	this.keyList = this.current.keyList;
	this.keyIndex = this.current.keyIndex;
	this.count = this.current.count;
}
BobsLib.StackTable.prototype.merge = function( ht ) {
	this.current.merge( ht );
	this.keyList = this.current.keyList;
	this.count = this.current.count;
	this.keyIndex = this.current.keyIndex;
}
BobsLib.StackTable.prototype.isUniqueKey = function( key ) {
	return this.keyIndex[ key ]  == undefined;
}
BobsLib.StackTable.prototype.toString = function() {
	return "[Object BobsLib.Stacktable] " + this.keyList;
}
///
/// 	Preferences 
///	Subclass of Hashtable - saves a Hashtable to a file, and reconstitutes it again
///	usage:
///		var prefs = new BobsLib.Preferences( fileObjectOrPath );
///		prefs.putValue( "filterString", "JPG,JPEG,TIF,TIFF" );
///		prefs.save();
///		var newPrefs = new BobsLib.Preferences( fileObjectOrPath ); // note same path as the first one!
///		newPrefs.load();
///		var filterString = newPrefs.getValue( "filterString" );
///
	BobsLib.Preferences = function( file, caseSensitive ) {
		this.ancestor = BobsLib.Hashtable;
		this.caseSensitive = caseSensitive || false;
		this.keyList = new Array( 0 );
		this.keyIndex = {};
		this.count = 0;
		this.file = new File( file ); 
		this.file.parent.verify();
		this.wasLoaded = false;
	}
	BobsLib.Preferences.prototype = new BobsLib.Hashtable();

	BobsLib.Preferences.prototype.load = function() {
		var oldLevel = $.level;
		$.level = 0;
		if ( this.file ) {
			if ( this.file.exists ) {
					this.file.open( "r" );
					var buf = this.file.read();
					this.file.close();
					var obj = eval( buf );
					this.merge( obj );
			} else {
				$.level = oldLevel;
				throw "Preferences Not Yet Saved";
			}
		} else {
			$.level = oldLevel;
			throw "Preferences File Not Set";
		} 
		$.level = oldLevel;
		this.wasLoaded = true;
	}
	BobsLib.Preferences.prototype.save = function() {
		if ( this.file ) {
			this.file.parent.verify();
			this.file.open( "w" );
			this.file.write( this.toSource() );
			this.file.close();
		} else {
			throw "Preferences File Not Set";
		}
	}
	BobsLib.Preferences.prototype.toString = function() {
		return "[Object BobsLib.Preferences] " + this.file.fsName;
	}
///
/// FIFOBuffer
///	obviously you can use an array, but this is just easier...
///
	BobsLib.FIFOBuffer = function() {
		this.ancestor = Array();
	}
	BobsLib.FIFOBuffer.prototype = new Array();
	BobsLib.FIFOBuffer.prototype.pop = function() {
		return this.shift();
	}
	BobsLib.FIFOBuffer.prototype.peek = function() {
		return this[ 0 ];
	}
	BobsLib.FIFOBuffer.prototype.toString = function() {
		return "[Object BobsLib.FIFOBuffer]" + this.toString();
	}
///
///	KeyedArrayList
///
///	put in a key and a value, it stores the value in an array, and ensures the array contains only unique values.
///	Only 1 array will be created for each key. This is handy when you're storing data in a number of arrays. With
///	this class, you can "name" the array with a key value, and reference it that way.
///	If sorted == true, the arrays will be sorted alphabetically ascending.
///
	BobsLib.KeyedArrayList = function( sorted ) {
		this.sorted = sorted || false;
		this.collection = new  BobsLib.Hashtable( true );
	}
	BobsLib.KeyedArrayList.prototype.putValue = function( key, value ) {
		if ( this.collection.isUniqueKey( key ) ) {
			var ar = new Array();
			ar.push( value );
			this.collection.put( key, ar );
		} else {
			ar = this.collection.get( key );
			for ( i = 0; i < ar.length; i++ ) {
				if ( ar[ i ] == value ) {
					return;
				}
			}
			ar.push( value );
			if ( this.sorted ) {
				ar.sort();
			}
		}
	}
	BobsLib.KeyedArrayList.prototype.getArray = function( key ) {
		return this.collection.getValue( key );
	}
	BobsLib.KeyedArrayList.prototype.getKeys = function() {
		return this.collection.keyList;
	}
///
///	Stack
///
	BobsLib.Stack = function() {
		this.stk = new Array( 0 );
		this.length = 0;
	}
	BobsLib.Stack.prototype.push = function( st ) {
		this.stk.push( st );
		this.length++;
	}
	BobsLib.Stack.prototype.pop = function() {
		this.length--;
		return this.stk.pop();
	}
	BobsLib.Stack.prototype.search = function( x )  {
		return BobsLib.getArrayIndex( this.stk, x );
	}
	BobsLib.Stack.prototype.toString = function() {
		return "[object BobsLib.Stack] " + this.stk.toString();
	}
	BobsLib.Stack.prototype.peek = function() {
		return this.stk[ ( this.stk.length - 1 ) ];
	}
///
// Extensions to Object Prototypes
///

///
/// 	Folder.verify()
///	ensures a Folder object, and the complete path to that folder exists. If you don't use the heck out of this one, you're stupid
/// 	usage: 
/// 		var f = new Folder( path );
/// 		f.verify(); - the folder pointed to by path will now exist or an error will be thrown
///
	Folder.prototype.verify = function() {
		if ( !this.exists ) { // if this folder doesn't exist
			var oFolder = new Folder( this.absoluteURI ); // make a new folder object (copy) to play with
			var stk = new Array();
			while ( !oFolder.exists ) { // we know this first one will not exist
				stk.push( oFolder ); // so push this folder onto the stack
					oFolder = new Folder( oFolder.path ); // and then check to see if the parent folder exists, if so, stop this loop
			}
			stk2 = new Array(); // this is to remove created folders if one of the subsequent folder creations fails - leave nothing behind
			while ( stk.length > 0 ) { // while there's still stuff in the stack
				oFolder = stk.pop(); // pop off the last one, create it, push it onto the error recovery stack if it works, if it doesn't unwind what we've done
				if ( oFolder.create() ) {
					stk2.push( oFolder );
				} else {
					while ( stk2.length > 0 ) {
						stk2.pop.remove();
					}
					throw "Folder creation failed";
				} 
			}
		}
	}

///
///	gets the extension of a file from the file name, option to NOT decode the file name
///
	File.prototype.getExtension = function( noDecode ) {
		var idx = noDecode ? this.name.lastIndexOf( "." ) :  decodeURIComponent( this.name ).lastIndexOf( "." );
		var ext = undefined;
		if ( idx > -1 ) {
			if ( noDecode ) {
				ext = this.name.substr( ( idx + 1 ) );
			} else {
				ext = decodeURIComponent( this.name ).substr( ( idx + 1 ) );
			}
		}
		return ext;
	}
///
/// gets the file name without the extension
///
	File.prototype.nameWithoutExtension = function( noDecode ) {
		var fName = noDecode ? this.name :  decodeURIComponent( this.name );
		var idx = fName.lastIndexOf( "." ); // could be more than 1 . in the file name
		if ( idx > -1 ) {
			fName = fName.substr( 0, idx );
		}
		return fName;
	}	
///
/// sets the file extension to a given string, note File Object must exist in the file system. This renames the actual disk file
///
	File.prototype.setExtension = function( ext ) {
		if ( this.exists ) {
			var newName = this.nameWithoutExtension( true ) + "." + ext;
			this.rename( newName );
		} else {
			throw "File Must Exist!";
		}
	}
///
///	File.getFileWithExtension - 
///
	File.prototype.getFileWithExtension = function( ext ) {
		var fName = this.nameWithoutExtension();
		return new File( this.path + "/" + fName + "." + ext );
	}
///
/// adds "(n)" to a file object's name until it is a unique file name
///
	File.prototype.uniquify = function() {
		var sq = 1;
		var oldName = this.nameWithoutExtension( true );
		var oldExt = this.getExtension( true );
		var aFile = this;
		while ( aFile.exists ) {
			aFile = new File( aFile.path + "/" + oldName + "(" + sq++ + ")." + oldExt );
		}
		return aFile;
	}
///
/// Array Extensions
///
	Array.prototype.intersection = function( a ) {
		var longer = a.length >= this.length ? a : this;
		var shorter = a.length < this.length ? a : this;
		var txt = "," + longer.toString() + ",";
		var answer = new Array();
		for ( var i in shorter ) {
			var grep = new RegExp( "," + shorter[ i ] + "," );
			if ( txt.match( grep ) != null ) {
				answer.push( shorter[ i ] );
			}
		}
		return answer;
	}
	Array.prototype.union = function ( a ) {
		var longer = a.length >= this.length ? a : this;
		var shorter = a.length < this.length ? a : this;
		var txt = "," + longer.toString() + ",";
		var answer = new Array();
		for ( var i in shorter ) {
			var grep = new RegExp( "," + shorter[ i ] + "," );
			if ( txt.match( grep ) == null ) {
				longer.push( shorter[ i ] );
			}
		}
		return longer;
	}
///
/// provide an extension or mac file type, or list of same. It will return true if the file is of that type
///
	File.prototype.isFileType = function( ex )  {
		if ( ! ex || ex == null ) {	return true;		}
		var ex = ex.toUpperCase();
		ext = this.getExtension();
		if ( !ext || ( ext == null ) ) {
			ext = "";
		}
		ext = BobsLib.trim(ext);

		// this is tricky now because OS X uses extensions
		// so if the extension can be used to determine type,
		// use it, otherwise try the type
		if (ex.indexOf( ext.toUpperCase() ) > -1) return true;
		// only try this on mac
		if ( File.fs == "Macintosh" ) {
			ext = ( this.type == '????' ) ? ext : this.type;
			return ex.indexOf( ext.toUpperCase() ) > -1;
		}
		// will be false at this point
		return false;
	}
///
/// String Functions
///
	BobsLib.equalsIgnoreCase = function( str, str1 )  {
		return str.toLowerCase() == str1.toLowerCase();
	}
	String.prototype.equalsIgnoreCase = function( str1 ) {
		return BobsLib.equalsIgnoreCase( this, str1 );
	}
	BobsLib.trimLeaveSpaces = function( str ) {
		str = new String( str ).replace( /^\n+/ , "" );
		str = new String( str ).replace( /\n+$/ , "" );
		str = new String( str ).replace( /^\r+/ , "" );
		str = new String( str ).replace( /\r+$/ , "" );
		str = new String( str ).replace( /^\t+/ , "" );
		str = new String( str ).replace( /\t+$/ , "" );
		str = new String( str ).replace( /^\f+/ , "" );
		str = new String( str ).replace( /\f+$/ , "" );
		str = new String( str ).replace( /^\v+/ , "" );
		str = new String( str ).replace( /^ +/ , " " );
		str = new String( str ).replace( / +/ , " " );
		return new String( str ).replace( /\v+$/ , "" );
	}
	BobsLib.removeExtraSpaces = function( str ) {
		return str.replace( / +/g, " " );
	}
	BobsLib.trimControlChars = function( str ) {
		str = new String( str ).replace( /^\n+/g , "" );
		str = new String( str ).replace( /\n+$/g , "" );
		str = new String( str ).replace( /^\r+/g , "" );
		str = new String( str ).replace( /\r+$/g , "" );
		str = new String( str ).replace( /^\t+/g , "" );
		str = new String( str ).replace( /\t+$/g, "" );
		str = new String( str ).replace( /^\f+/g , "" );
		str = new String( str ).replace( /\f+$/g , "" );
		str = new String( str ).replace( /^\v+/g , "" );
		return new String( str ).replace( /\v+$/g , "" );
	}
	BobsLib.trimAllControlChars = function( str ) {
		str = str.replace( /\n/g , "" );
		str = str.replace( /\r/g , "" );
		str = str.replace( /\t/g, "" );
		str = str.replace( /\f/g , "" );
		return str.replace( /\v/g , "" );
	}
	BobsLib.removeQuotes = function( str ) {
		return str.replace( /\"/g, "" );
	}
	BobsLib.ltrim = function( str )  {
		return new String( str ).replace( /^\s+/ , "" );
	}
	String.prototype.ltrim = function() {
		return BobsLib.ltrim( this );
	}
	BobsLib.rtrim = function( str )  {
		return new String( str ).replace( /\s+$/ , "" );
	}
	String.prototype.rtrim = function() {
		return BobsLib.rtrim( this );
	}
	BobsLib.trim = function( st )  {
		var str = new String( st );
		str = str.replace( /^\s+/ , "" );
		str = str.replace( /\s+$/ , "" );
		return str
	}
	BobsLib.unQuote = function( str ) {
		if ( str ) {
			var s = new String( str );
			if ( ( s.charAt( 0 ) == '"' ) && ( s.charAt( ( s.length - 1 ) ) == '"' ) )  {
				return s.substring( 1, ( s.length - 1 ) ).toString();			
			} else {
				return str;
			}
		} else {
			return undefined;
		}
	}
	String.prototype.trim = function() {
		return BobsLib.trim( this );
	}
	BobsLib.endsWith = function( str, end )  {
		return ( str.substr( ( str.length - end.length ), end.length ) == end );
	}
	String.prototype.endsWith = function( end ) {
		return BobsLib.endsWith( this, end );
	}
	BobsLib.startsWith = function( str, start )  {
		try {
			return ( str.substr( 0, start.length ) == start );
		} catch ( e ) {
			str = new String( str );
			return ( str.substr( 0, start.length ) == start );
		}
	}
	String.prototype.starsWith = function( start ) {
		return BobsLib.startsWith( this, start );
	}
	BobsLib.isEmpty = function( str )  {
		if ( !str ) {
			return true;
		}
		return ( str == null || str == "" )
	}	
	String.prototype.isEmpty = function() {
		return BobsLib.isEmpty( this );
	}
	BobsLib.contains = function( str, ch )  {
		return ( new String( str ).indexOf( ch ) > -1 );
	}
	BobsLib.isValidReference = function( ref )  {
	  return ( ref != undefined && ref != null );
	}
	BobsLib.replace = function( s1, s2, s3 )  {
		if ( BobsLib.isValidReference( s1 ) && BobsLib.isValidReference( s2 ) )	 {
			var grep = new RegExp( s2, "g" );
			if ( !BobsLib.isValidReference( s3) )  {
				s3 = "";
			}
			try {
				s1 = s1.replace( grep, s3 );
			} catch ( e ) {
				alert( e );
			}
		}
		return s1;
	}
	String.prototype.contains = function( ch ) {
		return BobsLib.contains( this, ch );
	}
	BobsLib.getCharsBefore = function( str, ch )  {
		s = new String( str );
		var idx = s.indexOf( ch );
		if (idx == -1 )  {
			return s.toString();
		} else {
			return s.substr( 0, idx );		
		}
	}
	String.prototype.getCharsBefore = function( ch ) {
		return BobsLib.getCharsBefore( this, ch );
	}
	BobsLib.getCharsAfter = function( str, ch )  {
		s = new String( str );
		var idx = s.indexOf( ch );
		if (idx == -1 )  {
			return "";
		} else {
			return s.substr( ( idx + ch.length ) );		
		}
	}
	String.prototype.getCharsAfter = function( ch ) {
		return BobsLib.getCharsAfter( this, ch );
	}

	BobsLib.getArrayIndex = function( arr, val )  {
		for ( var i = 0; i < arr.length ; i++ ) {
			if ( arr[i] == val )	 {
				return i;
			}
		}
		return -1;
	}
	BobsLib.arrayRemove = function( ar, idx )  {
		if ( idx > ar.length || idx < 0 ) {
			throw "Index out of range";
		}
		var nr = new Array( 0 );
		for ( var i = 0; i < ar.length ; i++ ) {
			if ( i != idx )  {
				nr.push( ar[ i ] );
			}
		}
		return nr;
	}
///
/// 	standard file type lists
/// 	these are not all inclusive lists, but do cover most of the possibilities for PS and AI
///	use these with the File.isFileType() method
///	if ( myFile.isFileType( BobsLib.fileFilters.getValue( "Photoshop Openable" ) ) ) {
///		[Do something really cool here because it CAN be opened by PS]
///	}
///
	BobsLib.fileFilterPath = Folder.userData + "/Adobe/Scripting/fileFilterPreferences.jsx";
	BobsLib.fileFilters = new BobsLib.Preferences( BobsLib.fileFilterPath );
	try {
		BobsLib.fileFilters.load();
	} catch( e ) {
		BobsLib.fileFilters.putValue( "All", "" );
		BobsLib.fileFilters.putValue( "Photoshop", "PSD,PDD,8BPS,JPEG,JPG,GIF," );
		BobsLib.fileFilters.putValue( "Photoshop Openable", "PSD,PDD,JPEG,JPG,JPE,8BPS,GIF,BMP,RLE,DIB,TIF,CRW,NEF,RAF,ORF," +
			"CIN,SDPX,DPX,FIDO,GIF,EPS,PS,EPSF,FLM,PSB,EXR,AI3,AI4,AI5,AI6,AI7,AI8,AI,PCX,PDF,PDP," +
			"PCD,RAW,PICT,PCT,PIC,PXR,PNG,HRR,RGBE,XYZE,TGA,VDA,ICB,VST,TIF,TIFF,WBM,WBMP,DNG,SCT,PBM," );
		BobsLib.fileFilters.putValue( "PageMaker", "PM6,PM6,P65,PMD,ALD6," );
		BobsLib.fileFilters.putValue( "InDesign",  "INDD,IDD3,IDD4,INDT," );
		BobsLib.fileFilters.putValue( "PDF", "pdf,PDF," );
		BobsLib.fileFilters.putValue( "Illustrator", "AI,AIT," );
		BobsLib.fileFilters.putValue( "Illustrator Openable",  "AI,AIT,PDF,DXF,BMP,RLE,CGM,CDR,EPS,EPSF,PS,EMF,FH?," +
			"GIF,JPEG,JPG,JPE,PCD,PCT,PIC,RTF,DOC,PCX,PSD,PDD,PXR,PNG,SVG,SVGZ,TGA,VDA,ICB,VST,TXT,TIF,TIFF,WMF," );
		BobsLib.fileFilters.putValue( "Contact Sheet", "JPG,JPEG,TIF,TIFF,GIF,AI,PSD,PDF,EPS,EPSF" );
		BobsLib.fileFilters.putValue( "GoLive", "HTML,HTM,CSS,JS" );
		BobsLib.fileFilters.putValue( "Script", "JSX" );
		BobsLib.fileFilters.save();
	}
///	
///	Utility Functions
/// 
	BobsLib.decimalToHex = function( d ) {
		var s = "00" + Number (d).toString (16);
		return s.substr (s.length - 2);	
	}
	BobsLib.hexToDecimal = function( hex ) {
		return parseInt( hex, 16);
	}
	BobsLib.round = function( number, precision ) {
		var number = parseFloat( number );
		var precision = parseInt( precision );
		if ( precision == 0 ) {
			return Math.round( number );
		}
		return Math.round( number * Math.pow( 10, precision ) ) / Math.pow( 10, precision );
	}
	BobsLib.isInteger = function( num ) {
		var iNum = parseInt( num );
		var rNum = parseFloat( num );
		if ( isNaN( iNum ) ) {
			return false;
		}
		return ( iNum == rNum );
	}
	BobsLib.withinPercent = function( num1, num2, pct ) {
		var pct = parseFloat( pct ) / 100;
		var num1 = parseFloat( num1 );
		var num2 = parseFloat( num2 );
		var loEnd = num2 - ( pct * num2 );
		var hiEnd = num2 + ( pct * num2 );
		return ( ( num1 <= hiEnd ) && ( num1 >= loEnd ) );
	}
///
/// createMenu - useful for creating menu elements in bridge
///
	BobsLib.createMenu = function( type, text, loc, id ) {
		var menu = MenuElement.find( id );
		if ( !menu ) {
			menu = MenuElement.create( type, text, loc, id );
		}
		return menu;
	}

/// functionName - setupDropdown 	- adds items to a dropdown or list boc
///
/// Description		If you store the choices for a ScriptUI dropdown in a Hashtable, this ditty
///				will take the dropdown list and Hashtable, and populate the list
/// arguments 		- list - ScriptUI list object (dropdown or list box )
///				- table = a hashtable - keys from the hashtable will be the items added
///				- selectedValue - if present will cause the key with the passed value to be the selected item
///				- selectedIndex - if present will cause the item with this index to be selected, note only works if
///                                                   selectedValue is passed as undefined, and will select the ADDED item by it's
///                                                   order in the hashtable. It will NOT select the list.item[ selectedIndex ]
/// returns		- Nothing
///
	BobsLib.setupDropdown = function( list, table, selectedText, selectedIndex ) {
		for ( var i in table.keyList ) {
			var item = list.add( "item", table.keyList[ i ]  );
			if ( selectedText ) {
				if ( table.keyList[ i ]  == selectedText ) {
					item.selected = true;
				}
			} else if ( selectedIndex != undefined ) {
				if ( i == selectedIndex ) {
					item.selected = true;
				}
			}
		}
	}
///
/// getGuid - if you ever needed a unique ID for something, this will do it 99.99% of the time.
///
	BobsLib.getGuid = function() {
			return ( new String( new Number( new Date() ) ) +":" + Math.round( 1000000 * Math.random() ) +  ":" + Math.round( 1000000 * Math.random() ) + ":" + Math.round( 1000000 * Math.random() ) );	
	}

"BobsScriptLibrary.jsx loaded";







	